﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using HeuristicLab.Analysis.AlgorithmBehavior.Analyzers;

namespace PerformanceTests {
  class Program {
    static void Main(string[] args) {
      // TestSimpleVolumeCalculation();
      //   TestVolCalc();
      //TestQHullVolumeCalculation();
      TestQhull();
    }

    public static void TestQhull() {
      var points = new List<double[]>();
      points.Add(new double[] { -0.72045174, 1.44667601, -1.75159125 });
      points.Add(new double[] { -0.67521503, 0.61127293, -0.33962646 });
      points.Add(new double[] { 2.02392510, -0.41962562, -1.12211942 });
      points.Add(new double[] { 0.02647963, -0.24983950, -0.47038916 });
      points.Add(new double[] { 0.61548803, -0.25039511, 0.97745744 });
      points.Add(new double[] { -1.65142294, -0.09537136, 1.93154268 });
      points.Add(new double[] { 1.12618422, 0.49384888, 0.05286691 });
      points.Add(new double[] { -0.12506782, -2.29161965, -0.09710301 });

      double volume = QhullWrapper.CalculateVolume(points);
      volume = Math.Round(volume, 6);
      Console.WriteLine("Volume is: " + volume);
    }


    public static void TestVolCalc() {
      int dim = 3;
      int size = 8;
      var data = CreateRandomData(size, dim);
      var dataList = data.ToList();
      Stopwatch watch = new Stopwatch();


      //calculate convex hull
      watch.Start();
      var convexHull = LPHull.Calculate(data);
      watch.Stop();
      Console.WriteLine("Convex hull calculation took : " + watch.Elapsed.TotalSeconds + " seconds.");


      //calculate volume of convex hull
      watch.Start();
      double volume = ConvexHullMeasures.CalculateVolume(convexHull);
      watch.Stop();
      Console.WriteLine("Volume calculation of convex hull took : " + watch.Elapsed.TotalSeconds + " seconds.");
      Console.WriteLine("Volume of convex hull is: " + volume);


      //do volume calculation from the whole data set
      watch.Start();
      volume = ConvexHullMeasures.CalculateVolume(dataList);
      watch.Stop();
      Console.WriteLine("Volume calculation of whole data set took : " + watch.Elapsed.TotalSeconds + " seconds.");
      Console.WriteLine("Volume of whole data is: " + volume);


      //do volume calculation with lrs
      watch.Start();
      volume = LiblrsWrapper.CalculateVolume(dataList);
      watch.Stop();
      Console.WriteLine("Volume calculation of whole data set with lrs took : " + watch.Elapsed.TotalSeconds + " seconds.");
      Console.WriteLine("Volume of whole data with lrs is: " + volume);
    }

    private static void TestQHullVolumeCalculation() {
      const int numPoints = 8;
      const int dimension = 3;

      var points = new List<double[]>();

      points.Add(new double[] { -0.72045174, 1.44667601, -1.75159125 });
      points.Add(new double[] { -0.67521503, 0.61127293, -0.33962646 });
      points.Add(new double[] { 2.02392510, -0.41962562, -1.12211942 });
      points.Add(new double[] { 0.02647963, -0.24983950, -0.47038916 });
      points.Add(new double[] { 0.61548803, -0.25039511, 0.97745744 });
      points.Add(new double[] { -1.65142294, -0.09537136, 1.93154268 });
      points.Add(new double[] { 1.12618422, 0.49384888, 0.05286691 });
      points.Add(new double[] { -0.12506782, -2.29161965, -0.09710301 });

      Stopwatch watch = new Stopwatch();
      //calculate convex hull
      watch.Start();
      var convexHull = LPHull.Calculate(points.ToArray());
      watch.Stop();
      Console.WriteLine("Convex hull calculation took : " + watch.Elapsed.TotalSeconds + " seconds.");


      //calculate volume of convex hull
      watch.Start();
      double volume = ConvexHullMeasures.CalculateVolume(convexHull);
      watch.Stop();
      Console.WriteLine("Volume calculation of convex hull took : " + watch.Elapsed.TotalSeconds + " seconds.");
      Console.WriteLine("Volume of convex hull is: " + volume);  //7.960698


      //do volume calculation from the whole data set
      watch.Start();
      volume = ConvexHullMeasures.CalculateVolume(points);
      watch.Stop();
      Console.WriteLine("Volume calculation of whole data set took : " + watch.Elapsed.TotalSeconds + " seconds.");
      Console.WriteLine("Volume of whole data is: " + volume);


      //do volume calculation with lrs
      watch = new Stopwatch();
      watch.Start();
      volume = LiblrsWrapper.CalculateVolume(points);
      watch.Stop();
      Console.WriteLine("Volume calculation of whole data set with lrs took : " + watch.Elapsed.TotalSeconds + " seconds.");
      Console.WriteLine("Volume of whole data with lrs is: " + volume);
    }

    private static void TestSimpleVolumeCalculation() {
      const int numPoints = 4;
      const int dimension = 3;

      double[] num1 = new double[dimension];
      double[] num2 = new double[dimension];
      double[] num3 = new double[dimension];
      double[] num4 = new double[dimension];

      num1[0] = 1;
      num1[1] = 2;
      num1[2] = 2;

      num2[0] = 3;
      num2[1] = 4;
      num2[2] = 3;

      num3[0] = 3;
      num3[1] = 6;
      num3[2] = 3;

      num4[0] = 7;
      num4[1] = 8;
      num4[2] = 3;

      var points = new List<double[]>();
      points.Add(num1);
      points.Add(num2);
      points.Add(num3);
      points.Add(num4);


      double volume = ConvexHullMeasures.CalculateSimplexVolume(points);

    }

    private static double[][] CreateRandomData(int n, int m) {
      double[][] result = new double[n][];
      Random rand = new Random();

      for (int i = 0; i < n; i++) {
        result[i] = new double[m];
        for (int j = 0; j < m; j++) {
          result[i][j] = (double)rand.Next(1, 60);
        }
      }
      return result;
    }
  }
}
